''
''   File...... SD_DS1307_DS1820_TV_XBEE_v103.spin
''   Purpose... Test Data Logger Program
'' 
''   Author.... Mike Cook
''   E-mail.... mike-cook@tx.rr.com 
''   Started... May 26, 2007
''   Updated... September 13, 2008
''   Compiler.. Parallax Propeller Tool v1.2
''   Platform.. Propeller Proto Board, Stock#: 32212    
''
''   Links..... SD Support:     http://tomas.rokicki.com/fat16work/
''                              http://forums.parallax.com/forums/default.aspx?f=25&m=162235
''              
''   Notes: Subsystem of the Pool Controller Project, Testing hardware and code operation, as of the Updated date, been running
''          for almost 9 months with no problems except for power outages.
''
''
''   History: DATE      Version    Notes:
''            =================================================================================================
''            09-14-08  v1.03      Added Day of the week to TV Display
''            09-13-08  v1.02      Updated TV_Text display output
''            01-17-08  v1.01      Setup Proto Board Prototype for long term testing
''            05-26-07  v1.00      Project Started
''
''
OBJ
  fsrw           : "fsrwFemto"                             ' FAT File System Read/Write
  term           : "TV_Text"
  DS1307         : "DS1307"
  Xbee           : "FullDuplexSerial"                      ' Declare Serial object for propeller to Xbee communication
  ow             : "OneWire"                               ' Define Dallas 1-Wire routines
  ' 
  ' 
CON
  _clkmode        = xtal1 + pll16x                         ' Run at 80 Mhz
  _xinfreq        = 5_000_000                              ' 5 MHz crystal
  '
  TvPins          = 12                                     ' TV interface starts on this pin and uses four if Broadcast is selected
  ' 
  spiDO           = 20                                     ' SD card installed on Pins 20..23
  spiClk          = 21
  spiDI           = 22
  spiCS           = 23
  '    
  RX_Pin          = 5                                      ' Input from Xbee PIN 2 ON RS-232
  TX_Pin          = 4                                      ' Output to Xbee PIN 3 ON RS-232
  BaudRate        = 9_600                                  ' Default Xbee baud rate 
  Mode            = %0000                                  ' Inverted RX & TX, Propeller has no RS-232 Driver chip,
  '                                                        ' Xbee does not have RS-232 driver chip (TTL to TTL)
  CR              = 13
  LF              = 10
  DEG             = $B0                                    ' degree symbol
  '
  DS1307_Addr     = %1101_0000
  i2cSCL          = 28
  i2cSDA          = 29
  '
  OW_DATA         = 0                                      ' 1-wire data pin
  '
  SKIP_ROM        = $CC                                    ' 1-wire commands
  MATCH_ROM       = $55
  READ_SCRATCHPAD = $BE
  CONVERT_T       = $44
  '
  ' Version Number displayed on TITLE Line
  MAJOR           = 1 + $30
  MINOR           = 3 + $30
  '
  '    
VAR
  long ioControl[2]                                        ' Needed for fsrwFemto
  word time                                                ' Used to hold minutes since midnight
  byte myTemp[9]                                           ' 9 Byte array to hold the DS1820 registers
  'byte fract
  'byte TempC
  byte month
  byte day
  byte year
  long loopCount
  '
  ' 
pub main | idx , newMinute, prevMinute, newDay, prevDay, r, DOW
  ' 
  StartUpXbee
  ow.start(OW_DATA)                                        ' start 1-wire object on pin define as OW_DATA     
  '
  fsrw.start(@ioControl)
  '
  if \fsrw.mount(spiDO,spiClk,spiDI,spiCS) < 0             ' Attempt to mount SD card
    abort term.str(@SdCardErr)                             '   if problem with SD card display card error
  ' 
  DS1307.init(DS1307_Addr, i2cSDA, i2cSCL,false)           ' setup the DS1307 clock  
  term.start(TvPins)
  '
  ' Used to manually load the date and time in the DS1307
  'ds1307.settime(07,45,10)                                 ' 07:45:10
  'ds1307.setdate(5,02,05,08)                               ' 02/05/2008 - day 5  
  '
  DS1307.getdate
  prevDay := julian(month, day, year)
  WriteFileName(DS1307.getYears,DS1307.getMonths,DS1307.getDays)
  '
  DS1307.gettime
  prevMinute := DS1307.getMinutes
  '
  ' Test Code
  {{' ----- START of Check julian -----
  idx := 1
  month := 12
  day := 31
  repeat 99
    xbee.dec(julian(month, day, idx))
    xbee.tx(",")
    xbee.dec((julian(month, day, idx) - 1) // 7)
    xbee.tx(CR)
    idx++
  repeat  
  }}' ----- END of Check julian -----
  '
  term.Str(@Title)   
  repeat
    SetXY(0,2)                                             ' loop forever
    ' get and display the DATE
    DS1307.getdate
    term.str(string("DS1307 Date           :     "))
    term.hex(DS1307.getMonths, 2)
    month := bcd2int(DS1307.getMonths)
    term.out("/")
    term.hex(DS1307.getDays, 2)
    day := bcd2int(DS1307.getDays)
    term.out("/")
    term.hex(DS1307.getYears, 2)
    year := bcd2int(DS1307.getYears)
    '    
    ' get and display the TIME
    SetXY(0,3)
    DS1307.gettime
    term.str(string("DS1307 Time           :     "))
    term.hex(DS1307.getHours, 2)
    time := bcd2int(DS1307.getHours) * 60                  ' the variable (time) holds the total minutes in a day
    '                                                      ' This line gets the hours and multiplies by 60 to convert
    '                                                      ' hours to total minutes
    term.out(":")
    term.hex(DS1307.getMinutes, 2)
    time += bcd2int(DS1307.getMinutes)                     ' then and minutes to the (time) variable, to get total
    '                                                      ' minutes since midnight
    term.out(":")
    term.hex(DS1307.getSeconds, 2)
    loopCount++
    '
    setXY(0,4)
    term.str(string("Day of the Week       :     "))
    setXY(28,4) 
    DOW := (julian(month, day, year)-1) // 7
    Case DOW
      0: term.str(string("Sunday   "))
      1: term.str(string("Monday   "))
      2: term.str(string("Tuesday  "))
      3: term.str(string("Wednesday"))
      4: term.str(string("Thursday "))
      5: term.str(string("Friday   "))
      6: term.str(string("Saturday "))
      OTHER: term.str(string("ERROR    "))    
    '
    ' Checks to see if it's a new Day, if so update the filename that data will be written to
    newDay := julian(month, day, year)
    if newDay ^ prevDay
      prevDay := newDay
      WriteFileName(DS1307.getYears,DS1307.getMonths,DS1307.getDays)
    '
    ' Checks to see if it's a new minute, if so sends this minute's data via RF and logs the data to the SD Card
    newMinute := DS1307.getMinutes
    if newMinute ^ prevMinute                              '
      prevMinute := newMinute
      '
      r := fsrw.popen(@FileName, "a")
      '
      SetXY(0,11)
      term.str(string("SD File I/O Status    :     "))
      term.dec(r)
      '
      ' Sample data that is transmitted via XBEE and written to SD CARD
      '
      ' DayOfWeek,JulianDate,MinSinceMidnight,TempA,TempB,CrLf
      '
      ' 6,39704,937,90F,93F
      '
      xbee.dec((julian(month, day, year)-1) // 7)
      xbee.tx(",")
      '
      fsrw.dec((julian(month, day, year)-1) // 7)
      fsrw.out(",")
      '
      xbee.dec(julian(month, day, year))
      xbee.tx(",")
      '
      fsrw.dec(julian(month, day, year))
      fsrw.out(",")
      ' 
      xbee.dec(time)
      xbee.tx(",")
      '
      fsrw.dec(time)
      fsrw.out(",")
      '
      xbee.dec(TempF(@ds1820a))                            ' this will be 'air' temperature
      xbee.str(string(DEG, "F"))
      xbee.tx(",")
      '
      fsrw.dec(TempF(@ds1820a))
      fsrw.str(string(DEG, "F")) 
      fsrw.out(",")
      '
      xbee.dec(TempF(@ds1820b))                            ' this will be 'water' temperature
      xbee.str(string(DEG, "F"))
      xbee.tx(CR)
      xbee.tx(LF)                                          ' END OF SERIAL TRANSMISSION
      '
      fsrw.dec(TempF(@ds1820b)) 
      fsrw.str(string(DEG, "F"))
      fsrw.out(CR)
      fsrw.out(LF)                                         ' END OF DATA RECORD FOR SD CARD
      '
      loopCount := 0
      fsrw.pclose                                          ' Close the Data log file 
    ' 
    ' debugging info sent to NTSC display
    SetXY(0,5)
    term.str(String("Minutes since Midnight:     "))
    term.dec(time)
    term.str(string("   "))
    '
    SetXY(0,6)
    term.str(string("Time variable MSB     :     "))
    term.dec(time / 256)
    SetXY(0,7)
    term.str(string("Time variable LSB     :     "))
    term.dec(time // 256)
    term.str(string("  "))
    term.out(CR)
    '
    SetXY(0,8)
    term.str(string("DS1820 Temp Sensor A  :     "))
    term.dec(TempF(@ds1820a))
    term.str(string(DEG, "F"))
    SetXY(0,9)
    term.str(string("DS1820 Temp Sensor B  :     ")) 
    term.dec(TempF(@ds1820b))
    term.str(string(DEG, "F"))
    '
    ' The code below is for debugging the Julian date routine
    SetXY(0,12)
    term.str(string("Julian Date debug data:     "))
    term.dec(month)
    term.out(" ")
    term.dec(day)
    term.out(" ")
    term.dec(year)
    term.out(" ")
    term.dec((month <# 3)/3)                               ' checks the month value for Feburary
    term.out(" ")
    term.dec(((year // 4) <# 1)+1)                         ' Checks the year for leap, 1 = Leap Year
    'term.str(string("          "))
    'term.dec(loopCount)
    '
    setXY(0,10)
    term.str(string("Data Log File Name    :          "))
    setXY(28,10)
    term.str(@filename)                                    ' Display name of file that is being written to
    ' End of Debug Code
  '     
  '  
  ' not used at this time......Used to boot from SD Card
  if \fsrw.mount(spiDO,spiClk,spiDI,spiCS) < 0             ' Attempt to mount SD card
    abort term.str(@SdCardErr)                             '   if problem with SD card display card error
  if \fsrw.popen(@FileName,"r")                            ' Attempt to open application file
    abort term.str(@SdFileErr)                             '   if application is not present then display file error
  fsrw.bootSDCard                                          ' All is good, boot application
  '
  '
PUB SetXY(x, y)
  ' Used to position the next printable position on the TV screen
  ' x = column, y = row
  '
  term.out($A)
  term.out(x) 
  term.out($B)
  term.out(y)
  return
  '
  '
PUB StartUpXbee
  ' XBEE RF modules works just like a wireless RS-232 cable in factory default mode.
  '
  Xbee.start(RX_Pin, TX_Pin, Mode, BaudRate)               ' Start serial port
  Xbee.rxflush                                             ' insure that the serial buffer is empty
  return
  '
  '
PUB Julian(MM, DD, YY) : JDN | JD, OFFSET
  '
  ' Returns the Julian Day Number; formatted for Microsoft Excel 2000, running on MS XP-PRO SP2
  '
  ' The following algorithm will work from dates from 2001 to 2099
  ' Original code from: http://www.emesys.com/BS2math4.htm#JulianDate
  ' visit the above URL for an explanation of the algorithm.
  '
  ' This subroutine calculates JD & JDN from mm/dd/yy (2001 to 2099), Returns JDN to caller. 
  ' MM, DD, YY are straight binary values, not BCD, return value is formatted for importing
  ' into Microsoft Excel 2000, running on MS XP-PRO SP2
  '
  '
  ' NOTE_1: Had to modify OFFSET by 1 to get it to work correctly with MS Excel 2000, running on XP-PRO SP2,
  '         same issue with code for Basic Stamp. Need to email original author to see if this was a typo
  '         on the web page or if I'm in error. Don't think this is in error, MSOffice and OpenOffice.org
  '         verifies that the date is correct when the number in the cell is converted to a date.
  '
  ' NOTE_2: When computing day of the week from JDN, modulo 7 reduces to day of the week. however looks like
  '         to get the DOW 'right' you need to subtract 1 from JDN and then modulo 7. Sunday=0, Monday=1,
  '         Tuesday=2,..,Saturday=6 for example: DOW := (julian(month, day, year)-1) // 7
  '
  ' NOTE_3: MAX in PBASIC == '<#' in Spin
  '
  '
  OFFSET := 36891                                          ' offset for Microsoft Excel, 36891 = December 31, 2000
  '                                                          Same for OpenOffice.org's Calc (Excel Clone)
  '
  ' PBASIC 2.5 Code
  '   JD=MM-1*30+(MM/9+MM/2)-(MM max 3/3*(YY//4 max 1 +1))+ DD
  ' Spin translation:
  JD := ((MM - 1) * 30 + (((MM / 9) + MM) / 2) - (((MM <# 3) / 3) * ((YY // 4 <# 1) + 1))) + DD
  '
  ' PBASIC 2.5 Code
  '   JDN=JD+(YY-1*365)+(YY-1/4)+offset
  ' Spin translation: 
  JDN := JD + ((YY - 1) * 365) + ((YY - 1) / 4) + OFFSET
  '
  return JDN 
  '
  '
PUB bcd2int(value) : result
  ' convert BCD (Binary Coded Decimal) to Integer
  '
  result :=((value / 16) *10) + (value // 16) 
  return result
  '
  '
PUB TempF(device) : result |  degC, degF
  '
  ' temperature is in fahrenheit, modified to only return whole number, all that's needed for this application     
  getTemperature(device)
  '
  ' convert back to 9-bit signed value
  degC := (myTemp[1] << 8) + mytemp[0] 
  '
  degC := degC << 23 ~> 23                                 ' extend sign bit
  degC *= 5                                                ' convert to 10ths
  degF := degC * 9 / 5 + 320                               ' convert to Fahrenheit
  result :=(degF / 10)
  return result
  '
  '
PUB getTemperature(device) | idx
  '
  ow.reset                                                 ' reset 1-Wire buss
  ow.writeByte(MATCH_ROM)                                  ' send match ROM command
  sendSerialNumber(device)                                 ' send serial number of the DS1820
  ow.writeByte(CONVERT_T)                                  ' send command to initiate a single temperature conversion
  '
  repeat                                                   ' wait for conversion
    'waitcnt(cnt + clkfreq / 1000 * 25)
    if ow.readBits(1)
      quit
  '  
  ow.reset                                                 ' reset 1-Wire buss
  ow.writeByte(MATCH_ROM)                                  ' send match ROM command  
  sendSerialNumber(device)                                 ' send serial number of the DS1820
  ow.writeByte(READ_SCRATCHPAD)                            ' send command to read the contents of the scratchpad
  idx := 0                                                 ' zero loop counter variable
  repeat 9                                                 ' read 9 bytes from 1-Wire buss 
    mytemp[idx++] := ow.readByte                           ' store byte in array
  return  
  '
  '
PUB sendSerialNumber(device) | idx, temp
  '
  idx := 0                                                 ' zero loop counter variable
  repeat 8                                                 ' read 8 bytes from data table
    temp := BYTE[device][idx++]                            ' byte from ds1820 data table
    ow.writeByte(temp)                                     ' write byte to 1-Wire buss
  return
  '
  '
PUB WriteFileName(YY,MM,DD)
  '
  ' modifies an existing zero terminated string named FileName, located in the DAT section.
  '
  '                     0         1
  '                     012345678901
  ' file name format is YY-MM-DD.LOG
  '
  fsrw.pclose
  WriteCharacter(YY <<= 24, 0)
  WriteCharacter(MM <<= 24, 3)
  WriteCharacter(DD <<= 24, 6)
  return
  '
  '
PUB WriteCharacter(char, idx)
  '
  ' writes a two character ASCII numeric digit to the position specified, of string data at FileName
  '
  repeat 2
    byte[@FileName][idx++] := lookupz((char <-= 4) & $F : "0".."9")
  return
  '
  '
DAT
  '
  SdCardErr  byte  "Can't Mount SD card", 0
  SdFileErr  byte  "Can't open file", 0
  '
  ' File name of the *.LOG file saved to the SD card   
  FileName   byte  "xx-xx-xx.LOG", 0
  '
  Title      byte  "SD & RF Temperature Data Logger     v", MAJOR, ".", MINOR, "----------------------------------------", 0                      
  '
  ' Serial number determined by using: OW-SearchDemo.spin program
  ds1820a    byte     $10, $1E, $1E, $E5, $00, $08, $00, $D3
  '
  ds1820b    byte     $10, $FF, $96, $C4, $00, $08, $00, $C7
  '
  ' 